package com.lzmispt.data.resource.service.datasync.impl.handle;

package .tmp;

import com.lzmispt.data.resource.entity.vo.datasync.DataSourceTableTreeVO;
import com.lzmispt.data.resource.service.datasync.impl.handle.AbstrcatDataSyncDatabase;
import com.lzmispt.dcbase.databasestruct.DataStructProvider;
import com.lzmispt.dcbase.dcconst.DCConst;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;
import org.springframework.stereotype.Service;

@Service
public class SqlServerDataSyncDataBaseHandler extends AbstrcatDataSyncDatabase {
  public List<DataSourceTableTreeVO> getDataBaseList(Long dataSourceId, String platCode, String tokenId, Boolean isEnableCdc) {
    DataStructProvider dataStructProvider = getDataStructProvider(dataSourceId, platCode, tokenId, null, null);
    List<String> sqlServerCdcDataBaseName = null;
    if (isEnableCdc.booleanValue()) {
      sqlServerCdcDataBaseName = dataStructProvider.getSqlServerCdcDataBaseName();
    } else {
      sqlServerCdcDataBaseName = dataStructProvider.getDataBaseNameList("sqlserver");
    } 
    List<DataSourceTableTreeVO> dataSourceTableTreeList = (List<DataSourceTableTreeVO>)sqlServerCdcDataBaseName.stream().map(dataBaseName -> buildDataSourceTableVO(dataBaseName, null, null, dataSourceId, dataStructProvider.getDbType(), DCConst.DirectoryType.DATABASE)).collect(Collectors.toList());
    return dataSourceTableTreeList;
  }
  
  public List<DataSourceTableTreeVO> getSchemaList(DataStructProvider dataStructProvider, Long dataSourceId, String type, String dataBaseName, Boolean isEnableCdc) {
    List<String> schemaNameList = new ArrayList<>();
    if (isEnableCdc.booleanValue()) {
      schemaNameList = dataStructProvider.getCdcEnabledSchemas(dataBaseName);
    } else {
      schemaNameList = dataStructProvider.getSchemaNameList("sqlserver", dataBaseName);
      schemaNameList = (List<String>)schemaNameList.stream().filter(schemaName -> !Objects.equals(schemaName, "cdc")).collect(Collectors.toList());
    } 
    return (List<DataSourceTableTreeVO>)schemaNameList.stream().map(schemaName -> buildDataSourceTableVO(dataBaseName, schemaName, null, dataSourceId, type, DCConst.DirectoryType.DATASCHEMA)).collect(Collectors.toList());
  }
  
  public List<DataSourceTableTreeVO> getTableList(DataStructProvider dataStructProvider, String schemaName, Long dataSourceId) {
    List<String> cdcEnabledTables = dataStructProvider.getCdcEnabledTables(schemaName);
    return (List<DataSourceTableTreeVO>)cdcEnabledTables.stream().map(tableName -> buildDataSourceTableVO(dataStructProvider.currentDataSource.getDbName(), schemaName, tableName, dataSourceId, dataStructProvider.getDbType(), DCConst.DirectoryType.DATATABLE))
      .collect(Collectors.toList());
  }
  
  public List<DataSourceTableTreeVO> getDataSourceTableTreeVO(Long dataSourceId, String platCode, String tokenId) throws Exception {
    DataStructProvider dataStructProvider = getDataStructProvider(dataSourceId, platCode, tokenId, null, null);
    List<DataSourceTableTreeVO> dataBaseList = getDataBaseList(dataSourceId, platCode, tokenId, Boolean.TRUE);
    return buildDataSourceTableTree(dataBaseList, dataStructProvider, platCode, tokenId);
  }
  
  private List<DataSourceTableTreeVO> buildDataSourceTableTree(List<DataSourceTableTreeVO> dataBaseList, DataStructProvider dataStructProvider, String platCode, String tokenId) {
    for (DataSourceTableTreeVO dataSourceTableTreeVO : dataBaseList) {
      dataStructProvider = getDataStructProvider(dataSourceTableTreeVO.getDataSourceId(), platCode, tokenId, dataSourceTableTreeVO.getSourceDatabaseName(), null);
      List<DataSourceTableTreeVO> dataSourceChild = getDataSourceChild(dataSourceTableTreeVO, dataStructProvider, platCode, tokenId);
      dataSourceTableTreeVO.setChildNodes(dataSourceChild);
    } 
    return dataBaseList;
  }
  
  private List<DataSourceTableTreeVO> getDataSourceChild(DataSourceTableTreeVO dataSourceTableTree, DataStructProvider dataStructProvider, String platCode, String tokenId) {
    List<DataSourceTableTreeVO> schemaList = getSchemaList(dataStructProvider, dataSourceTableTree.getDataSourceId(), dataStructProvider.currentDataSource.getType(), dataSourceTableTree.getDirectoryName(), Boolean.TRUE);
    schemaList.stream().forEach(schemaNameSourceTableTree -> {
          String schemaName = schemaNameSourceTableTree.getSourceDatabaseSchema();
          DataStructProvider getDataStructTable = getDataStructProvider(dataSourceTableTree.getDataSourceId(), platCode, tokenId, dataSourceTableTree.getSourceDatabaseName(), schemaNameSourceTableTree.getSourceDatabaseSchema());
          schemaNameSourceTableTree.setChildNodes(getTableList(getDataStructTable, schemaName, dataSourceTableTree.getDataSourceId()));
        });
    return schemaList;
  }
}
